﻿using System.IO;
using System.Windows.Media.Imaging;

namespace TestProject
{
    /// <summary>
    /// 쓰기 가능한 비트맵 확장
    /// </summary>
    public static class WriteableBitmapExtension
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 이미지 포맷 - ImageFormat

        /// <summary>
        /// 이미지 포맷
        /// </summary>
        public enum ImageFormat
        {
            /// <summary>
            /// 비트맵
            /// </summary>
            BMP,

            /// <summary>
            /// GIF
            /// </summary>
            GIF,

            /// <summary>
            /// JPG
            /// </summary>
            JPG,

            /// <summary>
            /// PNG
            /// </summary>
            PNG,

            /// <summary>
            /// TIFF
            /// </summary>
            TIFF,

            /// <summary>
            /// WMP
            /// </summary>
            WMP
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 저장하기 - Save(writeableBitmap, filePath, imageFormat)

        /// <summary>
        /// 저장하기
        /// </summary>
        /// <param name="writeableBitmap">쓰기 가능한 비트맵</param>
        /// <param name="filePath">파일 경로</param>
        /// <param name="imageFormat">이미지 포맷</param>
        public static void Save(this WriteableBitmap writeableBitmap, string filePath, ImageFormat imageFormat)
        {
            using(FileStream stream = new FileStream(filePath, FileMode.Create))
            {
                BitmapEncoder encoder = null;

                switch(imageFormat)
                {
                    case ImageFormat.BMP  : encoder = new BmpBitmapEncoder();  break;
                    case ImageFormat.GIF  : encoder = new GifBitmapEncoder();  break;
                    case ImageFormat.JPG  : encoder = new JpegBitmapEncoder(); break;
                    case ImageFormat.PNG  : encoder = new PngBitmapEncoder();  break;
                    case ImageFormat.TIFF : encoder = new TiffBitmapEncoder(); break;
                    case ImageFormat.WMP  : encoder = new WmpBitmapEncoder();  break;
                }

                encoder.Frames.Add(BitmapFrame.Create(writeableBitmap));

                encoder.Save(stream);
            }
        }

        #endregion
        #region 저장하기 - Save(writeableBitmap, filePath)

        /// <summary>
        /// 저장하기
        /// </summary>
        /// <param name="writeableBitmap">쓰기 가능한 비트맵</param>
        /// <param name="filePath">파일 경로</param>
        public static void Save(this WriteableBitmap writeableBitmap, string filePath)
        {
            FileInfo fileInfo = new FileInfo(filePath);

            switch(fileInfo.Extension.ToLower())
            {
                case ".bmp"  : writeableBitmap.Save(filePath, ImageFormat.BMP ); break;
                case ".gif"  : writeableBitmap.Save(filePath, ImageFormat.GIF ); break;
                case ".jpg"  :
                case ".jpeg" : writeableBitmap.Save(filePath, ImageFormat.JPG ); break;
                case ".png"  : writeableBitmap.Save(filePath, ImageFormat.PNG ); break;
                case ".tif"  :
                case ".tiff" : writeableBitmap.Save(filePath, ImageFormat.TIFF); break;
                case ".wmp"  : writeableBitmap.Save(filePath, ImageFormat.WMP ); break;
            }
        }

        #endregion
    }
}